/*
    Copyright (C) 2005, 2006  Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*******************************************************************************
    +++ UNIX/LINUX +++
    IO Layer
    ---------
        Specialized file/keyboard routines for linux.

                                                            Sunday, 4 dec 2005.
*******************************************************************************/
#include "../../logic/CHexEd.h"
#include <ncurses.h>
#include <ctype.h>
#include <sys/stat.h>
#include <errno.h>
#include <dirent.h>

ioLowLevel::ioLowLevel(){
}

ioLowLevel::~ioLowLevel(){
}

void ioLowLevel::DoEvents(){
}

int ioLowLevel::GetKey(){
    int ch;
        ch = getch();
        //log("ioLowLevel: key [%d]", ch);
        if(!isprint(ch) && (ch & KEY_CODE_YES)){
          //log("ioLowLevel: KEY_CODE_YES");
          switch(ch){
            case KEY_NPAGE:
            	    return (int)KPGD;
            case KEY_UP:
            	    return (int)KUP;
            case KEY_DOWN:
            	    return (int)KDOWN;
            case KEY_LEFT:
            	    return (int)KLEFT;
            case KEY_RIGHT:
            	    return (int)KRIGHT;
            case KEY_HOME:
                    return (int)KHOME;
        	case KEY_BACKSPACE:
            case KEY_DC:
        	    return KBACK;
            default:
                return (int)KNUL;
              break;
          };
        }else{
          if(ch == 27) return (int)KCANCEL;
          if (ch == KEY_PPAGE) return KPGU;
        }
    return ch;
}

void* ioLowLevel::ioOpenFile(char *path, int swRO){
    FILE *f = 0;
        errno = 0;
        f = fopen(path, (swRO ? "r" : "r+"));
        seterr(errno);
    return (void*)f;
}

void ioLowLevel::ioCloseFile(void* FileHandle){
    if(FileHandle) fclose((FILE*)FileHandle);
}

int ioLowLevel::ioIsEofFile(void* FileHandle){
    int res;
        errno = 0;
        res = feof((FILE*)FileHandle);
        seterr(errno);
    return res;
}

long ioLowLevel::ioReadFile(void* FileHandle, char *buffer, long amount){
    long bytes = 0;
        errno = 0;
        if(FileHandle) bytes = (long)fread(buffer, sizeof(char), amount, (FILE*)FileHandle);
        seterr(errno);
    return bytes;
}

long ioLowLevel::ioWriteFile(void* FileHandle, char *buffer, long amount){
    long bytes = 0;
        errno = 0;
        if(FileHandle) bytes = (long)fwrite(buffer, sizeof(char), amount, (FILE*)FileHandle);
        seterr(errno);
    return bytes;
}

void ioLowLevel::ioFlushFile(void* FileHandle){
    errno = 0;
    if(FileHandle) fflush((FILE*)FileHandle);
    seterr(errno);
}

int ioLowLevel::ioSeekFile(void* FileHandle, unsigned long FileAddress){
    int res = -1;
        errno = 0;
        if(FileHandle) res = fseek((FILE*)FileHandle, FileAddress, SEEK_SET);
        seterr(errno);
    return res;
}

unsigned long ioLowLevel::ioFileSize(void* FileHandle){
    struct stat st;
        errno = 0;
        memset(&st, 0, sizeof(struct stat));
        fstat(fileno((FILE*)FileHandle), &st);
        seterr(errno);
    return (unsigned long)st.st_size;
}


//Fills struct dir with files like in specified path, errno is returned
int ioLowLevel::ioListDirectory(ioDIR *structdir, char *path, char *like){
    DIRFILE df;
    errno = 0;
    DIR *dir = 0;
    struct dirent *namelist = 0;

        if(!structdir) return -1;
        structdir->Reset();
        if(!path || !strlen(path)) path = "/";
        sprintf(structdir->Directory, "%s", path);
        //log("structdir->Directory = '%s'", structdir->Directory);
        dir = opendir(structdir->Directory);
        if(dir){
            do{
                namelist = readdir(dir);
                if(namelist){
                    sprintf(df.FileName, namelist->d_name);
                    df.IsFolder = (namelist->d_type == DT_DIR ? 1 : 0);
                    structdir->AddElement(df);
                }
            }while(namelist);
            closedir(dir);
        }
        if(!structdir->CountElements()){
            sprintf(df.FileName, ".");
            df.IsFolder = 1;
            structdir->AddElement(df);
            sprintf(df.FileName, "..");
            df.IsFolder = 1;
            structdir->AddElement(df);
        }
    return errno;
}

char ioLowLevel::ioGetFileSeparatorChar(){ return '/'; }
