/*
    Copyright (C) 2005, 2006  Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*******************************************************************************
    +++ WINDOWS +++
    IO LAYER
    --------
        Specialized file/keyboard routines for Windows.

                                                            Friday, 6 jan 2006.
*******************************************************************************/
#include "../../logic/CHexEd.h"


ioLowLevel::ioLowLevel(){
}

ioLowLevel::~ioLowLevel(){
}

void ioLowLevel::DoEvents(){
    MSG msg;
        if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)){
            TranslateMessage(&msg);
    	   DispatchMessage(&msg);
    	}
}

int ioLowLevel::GetKey(){
    MSG msg;

    win_kbd = 0;
    while(GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
    	DispatchMessage(&msg);
    	if(win_kbd == 10 || win_kbd == 13) win_kbd = '\n';
    	if(win_kbd == 27) win_kbd = KCANCEL;
    	if(win_kbd) break;
    }

    return win_kbd;
}

void* ioLowLevel::ioOpenFile(char *path, int swRO){
    HANDLE hf = 0;
    DWORD   dwAccess = (swRO ? GENERIC_READ : GENERIC_READ + GENERIC_WRITE),
            dwShare = FILE_SHARE_READ | FILE_SHARE_WRITE,
            dwLerr = 0;

    hf = CreateFile( path,                   //LPCTSTR lpFileName,
                     dwAccess,               //DWORD dwDesiredAccess,
                     dwShare,                //DWORD dwShareMode,
                     0,                      //LPSECURITY_ATTRIBUTES lpSecurityAttributes,
                     OPEN_EXISTING,          //DWORD dwCreationDisposition,
                     0,  //DWORD dwFlagsAndAttributes,
                     0                       //HANDLE hTemplateFile
         );
    if(hf == INVALID_HANDLE_VALUE) hf = 0;
    dwLerr = GetLastError();
    if(hf) dwLerr = 0;
    seterr(dwLerr);
    SetLastError(dwLerr);
    //log("ioOpenFile: path = %s, HANDLE = 0x%X, LastError: %d", path, (HANDLE)hf, dwLerr);
  return (void*)hf;
}

void ioLowLevel::ioCloseFile(void* FileHandle){
    CloseHandle((HANDLE)FileHandle);
}

int ioLowLevel::ioIsEofFile(void* FileHandle){
    char ch[2];
    int res = 0;
        //read the file is 0 bytes returned the eof
        res = ioReadFile(FileHandle, ch, 1);
        if(res == 0)res = 1; else res = 0;
    return res;
}

long ioLowLevel::ioReadFile(void* FileHandle, char *buffer, long amount){
    DWORD bytes = 0, dwLerr = 0;
        if(!FileHandle) return 0;
        ReadFile(   (HANDLE)FileHandle,         //HANDLE hFile,
                    buffer,                     //LPVOID lpBuffer,
                    amount,                     //DWORD nNumberOfBytesToRead,
                    &bytes,                     //LPDWORD lpNumberOfBytesRead,
                    0                           //LPOVERLAPPED lpOverlapped
        );
        dwLerr = GetLastError();
        if(bytes > 0) dwLerr = 0;
        if(dwLerr == 183) dwLerr = 0;   //183: Cannot create a file when that file already exists.
        seterr(dwLerr);
        //log("ioReadFile: HANDLE = 0x%X, #bytes = %d, LastError: %d", (HANDLE)FileHandle, bytes, dwLerr);
    return (long)bytes;
}

long ioLowLevel::ioWriteFile(void* FileHandle, char *buffer, long amount){
    DWORD bytes = 0, dwLerr = 0;
        if(!FileHandle) return 0;

        WriteFile(  (HANDLE)FileHandle,         //HANDLE hFile,
                    buffer,                     //LPCVOID lpBuffer,
                    amount,                     //DWORD nNumberOfBytesToWrite,
                    &bytes,                     //LPDWORD lpNumberOfBytesWritten,
                    0                           //LPOVERLAPPED lpOverlapped
        );

        dwLerr = GetLastError();
        if(bytes > 0) dwLerr = 0;
        seterr(dwLerr);
    return (long)bytes;
}

void ioLowLevel::ioFlushFile(void* FileHandle){
  if(!FileHandle) return;
  FlushFileBuffers((HANDLE)FileHandle);
}

int ioLowLevel::ioSeekFile(void* FileHandle, unsigned long FileAddress){  //-1 = failed
    DWORD dwLerr;
    LARGE_INTEGER li;
        if(!FileHandle) return 0;
        li.QuadPart = FileAddress;
        SetFilePointer(  (HANDLE)FileHandle,
                         li.LowPart,
                         &li.HighPart,
                         FILE_BEGIN
        );
        dwLerr = GetLastError();
        if(dwLerr == 183) dwLerr = 0;   //183: Cannot create a file when that file already exists.
        seterr(dwLerr);
        //log("ioSeekFile: HANDLE = 0x%X, FileAddress = 0x%X, LastError: %d", (HANDLE)FileHandle, FileAddress, dwLerr);
    return (err() == 0 ? 0 : -1);
}

unsigned long ioLowLevel::ioFileSize(void* FileHandle){
    DWORD dwLerr;
    DWORD lo = 0, hi = 0;
    unsigned long sze = 0;

    if(!FileHandle) return 0;
    lo = GetFileSize(   (HANDLE)FileHandle,
                        &hi
        );
    sze = hi;
    sze = sze << 16;
    sze = sze | lo;

    dwLerr = GetLastError();
    if(sze > 0)dwLerr = 0;
    if(dwLerr == 183) dwLerr = 0;   //183: Cannot create a file when that file already exists.
    seterr(dwLerr);

    //log("ioFileSize: HANDLE = 0x%X, FileSize %d, LastError: %d", (HANDLE)FileHandle, sze, dwLerr);
    return sze;
}

#define MAXP 1024
char Pwd[MAXP];  //current directory
int ioLowLevel::ioListDirectory(ioDIR *dir, char *path, char *like){
    int sw_loop = 0;
    WIN32_FIND_DATA fd;
    HANDLE hFind;
    DIRFILE elem;
        if(!dir || !path) return -1;
        dir->Reset();
        if(!strlen(path)){
            memset(Pwd, 0, MAXP);
            if(!GetCurrentDirectory(MAXP, Pwd)) return (int)GetLastError();
            //sprintf(path, Pwd);
            path = Pwd;
        }
        sprintf(dir->Directory, path);
        sprintf(Pwd, "%s\\%s", path, like);
        memset(&fd, 0, sizeof(WIN32_FIND_DATA));
        hFind = FindFirstFile(Pwd, &fd);
        if(hFind != INVALID_HANDLE_VALUE){
            sw_loop = 1;
            while(sw_loop){
                sprintf(elem.FileName, fd.cFileName);
                if(fd.dwFileAttributes == FILE_ATTRIBUTE_DIRECTORY
                    ||
                   fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY
                )
                    elem.IsFolder = 1;
                else
                    elem.IsFolder = 0;
                dir->AddElement(elem);
                sw_loop = FindNextFile(hFind, &fd);
            }
            FindClose(hFind);
        }
        if(!dir->CountElements()){
            sprintf(elem.FileName, ".");
            elem.IsFolder = 1;
            dir->AddElement(elem);
            sprintf(elem.FileName, "..");
            elem.IsFolder = 1;
            dir->AddElement(elem);
        }
    return (int)GetLastError();
 }

char ioLowLevel::ioGetFileSeparatorChar(){ return '\\';}

