/*
    Copyright (C) 2005, 2006  Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*******************************************************************************
    +++ UNIX/LINUX +++
    PRESENTATION LAYER
    ------------------
        Specialized screen output routines, in occurence ncurses.

                                                            Sunday, 4 dec 2005.
*******************************************************************************/

#include "../../logic/CHexEd.h"
#include <sys/signal.h>
#include <ncurses.h>
#include <string.h>

void SIGNAL_RESIZE(int n){
    endwin();               //ncurses restore original screen here...
    refresh();
    attrset(WA_NORMAL);
    erase();
    flushinp();             //flush input
    hxe.DrawScreen();       //global hxe in main.cpp
}

uiLowLevel::uiLowLevel(){
    memset(this, 0, sizeof(uiLowLevel));
    InitScreen();
}

uiLowLevel::~uiLowLevel(){
    if(mWnd) endwin();
}

void uiLowLevel::InitScreen(){
    if(mWnd) endwin();
    mWnd = initscr();
    if(mWnd) {
       signal(SIGWINCH, SIGNAL_RESIZE);
	   noecho();
	   keypad(stdscr, TRUE);
	   use_default_colors();
    }
}

void uiLowLevel::commit(){
    refresh();
}

void uiLowLevel::clrscr(){
    attrset(WA_NORMAL);
    erase();
    move(0, 0);
    commit();
}

void uiLowLevel::clrln(){
    clrtoeol();
}

//ncurses screen surface
long uiLowLevel::GetMaxX(){
    return (long)COLS;
}

long uiLowLevel::GetMaxY(){
    return (long)LINES;
}

void uiLowLevel::cursor_on(int sw_turn){
}

void uiLowLevel::gotoxy(int x, int y){
    move(y, x);
}


void uiLowLevel::setDisplayModus(DISPLAY_MODUS modus){
    mModus = modus;
    switch(mModus){
        case DISP_REVERSE:
            attrset(WA_REVERSE);
            break;
        case DISP_HIGHLIGHTED:
            attrset(A_BOLD);
            break;
        case DISP_NORMAL:
            attrset(WA_NORMAL);
            break;
    };
}

void uiLowLevel::printchar(const char c){
    addch(c);
}

void uiLowLevel::print(const char *str){
     int c = 0, maxx = 0;
         maxx = GetMaxX();
         while(c < maxx && str[c]){
            addch(str[c]);
            c++;
         }
}


//safe formatted output
void uiLowLevel::printxy(int x, int y, const char *str){
    move(y, x);
    print(str);
}

void uiLowLevel::printcharxy(int x, int y, const char c){
    if(x > GetMaxX()) x = GetMaxX();
    if(y > GetMaxX()) y = GetMaxY();
    move(y, x);
    addch(c);
}

//converts a char to a '.' if it cannot be displayed on the screen
char uiLowLevel::GetDisplayChar(char ch){
    if( (unsigned int)ch >= ' '  && (unsigned int)ch <= ']'
        ||
        (unsigned int)ch >= '_' && (unsigned int)ch <= '}'
    ) ch = ch; else ch = '.';
    return ch;
}

